#ifndef TEXTURE_ENUMS_H
#define TEXTURE_ENUMS_H

/// <summary>
/// Enum of the various texture filters
/// </summary>
/// <remarks>
/// Values must match D3D12_FILTER
/// </remarks>
enum TextureFilters
{
  TEXTURE_FILTER_MIN_MAG_MIP_POINT                           = 0,
  TEXTURE_FILTER_MIN_MAG_POINT_MIP_LINEAR                    = 0x1,
  TEXTURE_FILTER_MIN_POINT_MAG_LINEAR_MIP_POINT              = 0x4,
  TEXTURE_FILTER_MIN_POINT_MAG_MIP_LINEAR                    = 0x5,
  TEXTURE_FILTER_MIN_LINEAR_MAG_MIP_POINT                    = 0x10,
  TEXTURE_FILTER_MIN_LINEAR_MAG_POINT_MIP_LINEAR             = 0x11,
  TEXTURE_FILTER_MIN_MAG_LINEAR_MIP_POINT                    = 0x14,
  TEXTURE_FILTER_MIN_MAG_MIP_LINEAR                          = 0x15,
  TEXTURE_FILTER_ANISOTROPIC                                 = 0x55,
  TEXTURE_FILTER_COMPARISON_MIN_MAG_MIP_POINT                = 0x80,
  TEXTURE_FILTER_COMPARISON_MIN_MAG_POINT_MIP_LINEAR         = 0x81,
  TEXTURE_FILTER_COMPARISON_MIN_POINT_MAG_LINEAR_MIP_POINT   = 0x84,
  TEXTURE_FILTER_COMPARISON_MIN_POINT_MAG_MIP_LINEAR         = 0x85,
  TEXTURE_FILTER_COMPARISON_MIN_LINEAR_MAG_MIP_POINT         = 0x90,
  TEXTURE_FILTER_COMPARISON_MIN_LINEAR_MAG_POINT_MIP_LINEAR  = 0x91,
  TEXTURE_FILTER_COMPARISON_MIN_MAG_LINEAR_MIP_POINT         = 0x94,
  TEXTURE_FILTER_COMPARISON_MIN_MAG_MIP_LINEAR               = 0x95,
  TEXTURE_FILTER_COMPARISON_ANISOTROPIC                      = 0xd5,
  TEXTURE_FILTER_MINIMUM_MIN_MAG_MIP_POINT                   = 0x100,
  TEXTURE_FILTER_MINIMUM_MIN_MAG_POINT_MIP_LINEAR            = 0x101,
  TEXTURE_FILTER_MINIMUM_MIN_POINT_MAG_LINEAR_MIP_POINT      = 0x104,
  TEXTURE_FILTER_MINIMUM_MIN_POINT_MAG_MIP_LINEAR            = 0x105,
  TEXTURE_FILTER_MINIMUM_MIN_LINEAR_MAG_MIP_POINT            = 0x110,
  TEXTURE_FILTER_MINIMUM_MIN_LINEAR_MAG_POINT_MIP_LINEAR     = 0x111,
  TEXTURE_FILTER_MINIMUM_MIN_MAG_LINEAR_MIP_POINT            = 0x114,
  TEXTURE_FILTER_MINIMUM_MIN_MAG_MIP_LINEAR                  = 0x115,
  TEXTURE_FILTER_MINIMUM_ANISOTROPIC                         = 0x155,
  TEXTURE_FILTER_MAXIMUM_MIN_MAG_MIP_POINT                   = 0x180,
  TEXTURE_FILTER_MAXIMUM_MIN_MAG_POINT_MIP_LINEAR            = 0x181,
  TEXTURE_FILTER_MAXIMUM_MIN_POINT_MAG_LINEAR_MIP_POINT      = 0x184,
  TEXTURE_FILTER_MAXIMUM_MIN_POINT_MAG_MIP_LINEAR            = 0x185,
  TEXTURE_FILTER_MAXIMUM_MIN_LINEAR_MAG_MIP_POINT            = 0x190,
  TEXTURE_FILTER_MAXIMUM_MIN_LINEAR_MAG_POINT_MIP_LINEAR     = 0x191,
  TEXTURE_FILTER_MAXIMUM_MIN_MAG_LINEAR_MIP_POINT            = 0x194,
  TEXTURE_FILTER_MAXIMUM_MIN_MAG_MIP_LINEAR                  = 0x195,
  TEXTURE_FILTER_MAXIMUM_ANISOTROPIC                         = 0x1d5
};

/// <summary>
/// Enum of the various texture address modes
/// </summary>
/// <remarks>
/// Values must match D3D12_TEXTURE_ADDRESS_MODE
/// </remarks>
enum TextureAddressMode
{
  TEXTURE_ADDRESS_MODE_WRAP = 1,
  TEXTURE_ADDRESS_MODE_MIRROR,
  TEXTURE_ADDRESS_MODE_CLAMP,
  TEXTURE_ADDRESS_MODE_BORDER,
  TEXTURE_ADDRESS_MODE_MIRROR_ONCE
};

/// <summary>
/// Enum of the various texture border colors
/// </summary>
/// <remarks>
/// Values must match D3D12_STATIC_BORDER_COLOR
/// </remarks>
enum TextureBorderColor
{
  BORDER_COLOR_TRANSPARENT_BLACK = 0,
  BORDER_COLOR_OPAQUE_BLACK,
  BORDER_COLOR_OPAQUE_WHITE
};

#endif /* TEXTURE_ENUMS_H */
